package org.zeroturnaround.javarebel.integration.servlet;

import org.zeroturnaround.javarebel.integration.servlet.util.IoUtil;
import org.zeroturnaround.javarebel.integration.servlet.util.ServletUtil;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.zeroturnaround.javarebel.Logger;
import org.zeroturnaround.javarebel.LoggerFactory;

/**
 * Servlet that provides resources using the <code>ServletContext</code>.
 * <p>
 * The purpose of this servlet is to provide resources accessed through HTTP
 * by using the <code>ServletContext</code> interface instead of any
 * container specific mechanisms. This enables JavaRebel to handle all
 * resources accessed through HTTP independent of the specific container.
 * </p>
 * <p>
 * This servlet should be mapped as the default servlet:
 * <pre>
 * &lt;servlet&gt;
 *   &lt;servlet-name&gt;defaultServlet&lt;/servlet-name&gt;
 *   &lt;servlet-class&gt;org.zeroturnaround.javarebel.integration.servlet.JavaRebelResourceServlet&lt;/servlet-class&gt;
 * &lt;/servlet&gt;
 *
 * &lt;servlet-mapping&gt;
 *   &lt;servlet-name&gt;defaultServlet&lt;/servlet-name&gt;
 *   &lt;url-pattern&gt;/&lt;/url-pattern&gt;
 * &lt;/servlet-mapping&gt;
 * </pre>
 * 
 * @author Rein Raudjärv
 * 
 * @see ServletContext
 */
public class JavaRebelResourceServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;
	
	private static final Logger log = LoggerFactory.getInstance();

	// TODO parse web.xml
	private String[] welcomeFiles = { "welcome.html", "index.html", "index.htm", "index.jsp" };

	public void init() {
		log.log("JavaRebelResourceServlet initialized");
	}
	
	public void destroy() {
		log.log("JavaRebelResourceServlet destroyed");
	}

	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
		try {
			process(req, resp);
		}
		catch (Exception e) {
			throw new ServletException(e.getMessage(), e);
		}
	}

	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
		try {
			process(req, resp);
		}
		catch (Exception e) {
			throw new ServletException(e.getMessage(), e);
		}
	}
	
	private void process(HttpServletRequest req, HttpServletResponse resp) throws Exception {
		
		String contextPath = req.getContextPath();
		String path = ServletUtil.getUriFromRequest(req);
		
		ServletContext servletContext = getServletContext();

		if (isDirectory(servletContext, path)) {
			if (!path.endsWith("/")) {
				// Trying to add '/' to end
				path += "/";

				resp.setContentLength(0);
				resp.sendRedirect(contextPath + path);
				return;
			}
			// path ends with /

			// Redirect to a welcome file
			for (int i = 0; i < welcomeFiles.length; i++) {
				String welcomeFile = welcomeFiles[i];
				if (exists(servletContext, path + welcomeFile)) {
					// Found an existing welcome file
					resp.setContentLength(0);
					resp.sendRedirect(contextPath + path + welcomeFile);
					return;
				}
			}
			// No welcome files found

			// Show directory listing
			String buf = sendDirectory(contextPath, path, servletContext.getResourcePaths(path));
			resp.getWriter().write(buf);
			return;
		}
		else {
			InputStream in = servletContext.getResourceAsStream(path);
			if (in == null) {
				log.log("JavaRebelResourceServlet: Resource not found: " + contextPath + path);
				resp.sendError(HttpServletResponse.SC_NOT_FOUND);
				return;
			}

			log.log("JavaRebelResourceServlet: Sending data: " + contextPath + path);
			sendData(req, resp, in);
			return;
		}
	}

	private static boolean isDirectory(ServletContext servletContext, String path) {
		if ("".equals(path) || "/".equals(path))
			return true;

		// Remove the / from the end
		String p = path;
		if (p.endsWith("/"))
			p = p.substring(0, p.length() - 1);

		Set paths = servletContext.getResourcePaths(p);
		if (paths == null || paths.isEmpty())
			return false;

		String result = (String) paths.iterator().next();
		return result.equals(p + "/");
	}

	private static boolean exists(ServletContext servletContext, String path) {
		try {
			return servletContext.getResource(path) != null;
		} catch (MalformedURLException e) {
			return false;
		}
	}

	private static void sendData(HttpServletRequest req, HttpServletResponse resp, InputStream in) throws Exception {
		OutputStream out = resp.getOutputStream();

		try {
			IoUtil.copy(in, out);
		}
		finally {
			in.close();
		}
	}

	private static String sendDirectory(String contextPath, String path, Set resourcePaths) throws IOException {
		// TODO encode/decode URLs, see method org.mortbay.resource.Resource.getListHTML
		
		log.log("JavaRebelResourceServlet: Sending directory listing: " + contextPath + path);
		
		StringBuffer buf = new StringBuffer(4096);
			
		String decodedBase = contextPath + path;
		String title = "Directory: " + decodedBase;
		
		buf.append("<HTML><HEAD><TITLE>");
		buf.append(title);
		buf.append("</TITLE></HEAD><BODY>\n<H1>");
		buf.append(title);
		buf.append("</H1><TABLE BORDER=0>");
		
		{
			String href = contextPath + path;
			// Remove / from the end
			href = href.substring(0, href.length() - 1);
			// Remove last directory from the path
			href = href.substring(0, href.lastIndexOf("/") + 1);
			
			buf.append("<TR><TD><A HREF=");
			buf.append(href);
			buf.append(">Parent Directory</A></TD><TD></TD></TR>\n");
		}
		
		Collection paths = new TreeSet(resourcePaths);
		for (Iterator it = paths.iterator(); it.hasNext();) {
			String subPath = (String) it.next();
			String href = contextPath + subPath; 
			
			buf.append("<TR><TD><A HREF=\"");
			buf.append(href);
			
			buf.append("\">");
			buf.append(subPath);
			buf.append("&nbsp;");
			buf.append("</TD><TD ALIGN=right>");
			buf.append("</TD></TR>\n");
		}
		
		buf.append("</TABLE>\n");
		buf.append("</BODY></HTML>\n");
		
		return buf.toString();
	}

}
