package org.zeroturnaround.javarebel.integration.servlet.weblogic;


import org.zeroturnaround.javarebel.integration.servlet.util.IoUtil;
import org.zeroturnaround.javarebel.integration.servlet.util.ServletUtil;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import org.zeroturnaround.javarebel.LoggerFactory;

/**
 * JavaRebel filter for handling JSP pages in BEA/Oracle Weblogic.
 * <p>
 * As Weblogic extracts the JSPS to a temporary folder they will be read only from there.
 * This filter synchronizes the temporary directory with resources found from the
 * <code>ServletContext</code>.
 * </p>
 * <p>
 * The filter works only with domains in development mode not in production mode
 * (in the latter case the temporary directory is not being looked for the updates).
 * 
 * @author Rein Raudjärv
 */
public class WeblogicJspUpdateFilter implements Filter {

	private boolean weblogicFound;
	
	private ServletContext context;
	private File warTempDir;

	public void init(FilterConfig config) throws ServletException {
		context = config.getServletContext();
		
		try {
			File rootTempDir = getRootTempDir(context);
			warTempDir = new File(rootTempDir, "war");
			weblogicFound = true;
		} catch (ClassNotFoundException e) {
		    LoggerFactory.getInstance().echo("JavaRebel: WeblogicJspUpdateFilter did not find Weblogic servlet context - filter is disabled.");
			// weblogicFound = false
		}
	}

	public void doFilter(ServletRequest req, ServletResponse resp, FilterChain chain) throws IOException, ServletException {
		if (weblogicFound) {
			String path = ServletUtil.getUriFromRequest((HttpServletRequest) req);
			
			String lc = path.toLowerCase();
			if (lc.endsWith(".jsp") || lc.endsWith(".jspx")) {
				String realPath = context.getRealPath(path);
				if (realPath == null) {
				    LoggerFactory.getInstance().echo("JavaRebel: WeblogicJspUpdateFilter could not get real path for '" + path + "' - filter is disabled.");
					weblogicFound = false;
				}
				else {
					File tempFile = new File(warTempDir, path);
					File realFile = new File(realPath);
					
					long tempDate = tempFile.lastModified();
					long realDate = realFile.lastModified();
					
					if (realDate > tempDate) {
						IoUtil.copy(realFile, tempFile);
						LoggerFactory.getInstance().echo("JavaRebel: JSP file '" + path + "' updated.");
					}
				}
			}
		}
		chain.doFilter(req, resp);
	}

	public void destroy() {
	}

	private static File getRootTempDir(ServletContext context) throws ClassNotFoundException {
		Class type = Class.forName("weblogic.servlet.internal.WebAppServletContext");

		if (!type.isInstance(context)) {
			throw new IllegalArgumentException("Unexpected servlet context type: " + context.getClass().getName());
		}

		Method method = null;
		try {
			method = type.getMethod("getRootTempDir", new Class[0]);
		} catch (SecurityException e) {
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		}

		File result = null;
		try {
			result = (File) method.invoke(context, new Object[0]);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		}

		return result;
	}

}
